﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=6.01
@EndOfDesignText@
'Static code module
Sub Process_Globals
	Private fx As JFX
	Private Project As ProjectType
	Private ProjectCount As Int
	Private StartTime As Long
	Private FolderList As List
	Type ScanFile(L As List,Index As Int,Root As Boolean)
	
	Public Const GitRepo As String = ".git"
End Sub

Public Sub ScanFolder(Folder As String,Root As Boolean)

	FolderList.Initialize
	ProjectCount = 0
	StartTime = DateTime.Now
	
	ScanFolderDo(Folder, Root)
	
	Wait For Scan_Complete
	
	CallSub(Main,"Scan_Complete")
	If ProjectCount = 0 Then
		fx.Msgbox(Main.MainForm,"No project found in " & Folder,"Project not found")
	Else
		fx.Msgbox(Main.MainForm,ProjectCount & " Projects parsed in " & ((DateTime.Now - StartTime) / 1000) & " Seconds","Scan Complete")
	End If
End Sub

Public Sub ScanFolderDo(Folder As String,Root As Boolean)
	FolderList.Add(Folder)
	ScanNext
End Sub

Private Sub ScanNext
	If FolderList.Size > 0 Then
		Dim Folder As String = FolderList.Get(0)
		FolderList.RemoveAt(0)
		Dim Files As List = File.ListFiles(Folder)
		ScanProject(Folder,Files)
	Else
		CallSubDelayed(Me,"Scan_Complete")
	End If
	
End Sub

Private Sub ScanProject(Folder As String, Files As List)
	Dim Git1 As Boolean
	For Each FN As String In Files
		If (FN = "Objects" Or FN = "Files") And File.IsDirectory(Folder,FN) Then Continue
		If FN = GitRepo Then
			Git1 = True
			Continue
		End If
		If File.IsDirectory(Folder,FN) Then
			FolderList.Add(File.Combine(Folder,FN))
			Continue
		End If
		If FN.EndsWith(".b4j") Or FN.EndsWith(".b4a") Or FN.EndsWith(".b4i") Or FN.EndsWith(".b4r") Then
			Main.lblShowingCount.Text = FN
			Sleep(0)
			ProjectCount = ProjectCount + 1
			ParseProjectFile(Folder,FN,Git1)
			If Project.Modules.Size > 0 Then
				Sleep(0)
				ParseModules
			End If
			Sleep(0)
			DBUpdates.UpdateProject(Project)
			Sleep(0)
		End If
	
	Next
	ScanNext
End Sub

Private Sub ParseProjectFile(Folder As String, Fn As String, HasGit As Boolean)
	Project  = NewProject(Folder,Fn)
	Project.Git = HasGit
	Project.LastModified = File.LastModified(Folder,Fn)
	
'	Log("Parse Project " & Folder & " : " &  Fn)
	Dim LineList As List = File.ReadList(Folder,Fn)
	Dim LineNo As Int = 0
	Dim Line As String = ""
	
	Dim MainModule As ModuleType = NewModule(Folder,"Main")
	MainModule.FileSize = File.Size(Folder,Fn)
	MainModule.LastModified = Project.LastModified
	
	Project.Modules.Add(MainModule)
	
	Line = LineList.Get(LineNo)
	Line = Line.Trim
	Do While Line <> "@EndOfDesignText@"
		If Line.StartsWith("FileGroup") Or Line.StartsWith("Group") Then
			'Do Nothing, just make sure it's not added to the Files list
		Else If Line.StartsWith("File") Then
			Project.Files.Add(GetValue(Line))
		Else If Line.StartsWith("Module") Then
			Dim Module As ModuleType = NewModule(Folder,GetValue(Line))
			Project.Modules.Add(Module)
		Else If Line.StartsWith("Library") Then
			Project.Libraries.Add(GetValue(Line))
		Else If Line.StartsWith("ManifestCode") Then
			Project.Permissions.AddAll(GetPermissions(Line))
		End If
		LineNo = LineNo + 1
		If LineNo >= LineList.Size Then Exit
		Line = LineList.Get(LineNo)
		Line = Line.Trim
	Loop
	
	Dim SubFound As Boolean
	
	Do While True
		If Line.ToLowerCase.StartsWith("private sub ") Or Line.ToLowerCase.StartsWith("public sub ") Or Line.ToLowerCase.StartsWith("sub ") Then
			MainModule.Subs.Add(GetSubName(Line))
			MainModule.CodeLines = MainModule.CodeLines + 1
			SubFound = True
		Else If Line.ToLowerCase.StartsWith("#additionaljar") Then
			Project.AddlJars.Add(GetAttributeValue(Line))
			MainModule.CodeLines = MainModule.CodeLines + 1
		Else If Line.ToLowerCase.StartsWith("#virtualmachineargs") Then
			Project.VirtualMachineArgs.Add(GetAttributeValue(Line))
			MainModule.CodeLines = MainModule.CodeLines + 1
		Else If Line.StartsWith("'") Then
			MainModule.CommentLines = MainModule.CommentLines + 1
			If Not(SubFound) Then
				MainModule.Description = MainModule.Description & CRLF & Line
			End If
		Else If Line <> "" Then
			MainModule.CodeLines = MainModule.CodeLines + 1
		End If
		
		If Line.ToLowerCase.Contains("initializenewinstance(") Then
			Dim Class As String = GetJavaClass(Line)
			If Class <> "" And Class.ToLowerCase <>"initializenewinstance("  And MainModule.JavaClasses.IndexOf(Class) = -1 Then MainModule.JavaClasses.Add(Class)
		End If
		If Line.ToLowerCase.Contains("initializestatic(") Then
			Dim Class As String = GetJavaClassStatic(Line)
			If Class <> "" And Class.ToLowerCase <> "initializestatic(" And MainModule.JavaClasses.IndexOf(Class) = -1 Then MainModule.JavaClasses.Add(Class)
		End If
		If Line.ToLowerCase.Contains("createevent") Then
			Dim Class As String = GetJavaEvent(Line)
			If Class <> "" And MainModule.JavaEvents.IndexOf(Class) = -1  Then MainModule.JavaEvents.Add(Class)
		Else If Line.ToLowerCase.Contains(".addchangelistener(") Then
			Dim Class As String = "javafx.beans.value.ChangeListener"
			Dim Result As String = Class & " on " & GetParam(Line,1)
			If Class <> "" And MainModule.JavaEvents.IndexOf(Result) = -1  Then MainModule.JavaEvents.Add(Result)
		Else If Line.ToLowerCase.Contains(".addeventfilter(") Then
			Dim Class As String = GetParam(Line,1)
			If Class <> "" And MainModule.JavaEvents.IndexOf(Class) = -1  Then MainModule.JavaEvents.Add(Class)
		Else If Line.ToLowerCase.Contains(".addeventhandler2(") Then
			Dim Class As String = GetParam(Line,1)
			If Class <> "" And MainModule.JavaEvents.IndexOf(Class) = -1  Then MainModule.JavaEvents.Add(Class)
		Else If Line.ToLowerCase.Contains(".addeventhandler(") Then
			Dim Class As String = GetParam(Line,1)
			If Class <> "" And MainModule.JavaEvents.IndexOf(Class) = -1  Then MainModule.JavaEvents.Add(Class)
		Else If Line.ToLowerCase.Contains(".createobject2(") Then
			Dim Class As String = GetJavaClass(Line)
			If Class <> "" And Class.ToLowerCase <> "createobject2("  And MainModule.JavaClasses.IndexOf(Class) = -1 Then MainModule.JavaClasses.Add(Class)
		Else If Line.ToLowerCase.Contains(".createobject(") Then
			Dim Class As String = GetJavaClassStatic(Line)
			If Class <> "" And Class.ToLowerCase <> "createobject("  And MainModule.JavaClasses.IndexOf(Class) = -1 Then MainModule.JavaClasses.Add(Class)
		Else If Line.ToLowerCase.StartsWith("type ") Then
			MainModule.Types.Add(Line.SubString(5))
		Else If Line.ToLowerCase.EndsWith("as form") Then
			MainModule.Forms.Add(GetWord(Line,1))
		Else If Line.StartsWith("#LibraryName:") Then
			Project.LibraryName = GetAttributeValue(Line)
		Else If Line.StartsWith("#LibraryVersion:") Then
			Project.LibraryVersion = GetAttributeValue(Line)
		End If
		LineNo = LineNo + 1
		If LineNo >= LineList.Size Then Exit
		Line = LineList.Get(LineNo)
		Line = Line.Trim
	Loop
	
	If MainModule.Description <> "" Then MainModule.Description = MainModule.Description.SubString(1)
	
End Sub

Private Sub GetPermissions(Line As String) As List
	Dim L As List
	L.Initialize
	Dim Lines() As String = Regex.Split("\\n",Line)
	For Each LLine As String In Lines
		If LLine.Contains("AddPermission(") Then
			L.Add(GetParam(LLine,0).Replace(QUOTE,""))
		End If
	Next
	Return L
End Sub

Private Sub GetParam(Line As String,Index As Int) As String
	Dim Params() As String = Regex.Split(",",Utils.GetEnclosedSingle("(",")",Line))
	If Index < Params.Length Then Return(Params(Index))
	Return ""
End Sub

Private Sub GetJavaClass(Line As String) As String
	Return Utils.GetEnclosedSingle($"(""$,$"","$,Line)
End Sub

Private Sub GetJavaEvent(Line As String) As String
	Return Utils.GetEnclosedSingle($"(""$,$"","$,Line)
End Sub

Private Sub GetJavaClassStatic(Line As String) As String
	Return Utils.GetEnclosedSingle($"(""$,$"")"$,Line)
End Sub

Private Sub GetWord(Line As String,Index As Int) As String
	Line = Line.Replace(TAB, " ")
	Do While Line.Contains("  ")
		Line = Line.Replace("  ", " ")
	Loop
	Dim Words() As String = Regex.Split(" ",Line)
	If Index > Words.Length - 1 Then Return ""
	Return Words(Index)
End Sub

Private Sub ParseModules
	
	For Each Module As ModuleType In Project.Modules
'		Log("Module " & Module.FileName)
		
		If Module.ModuleName = "Main" Then Continue
		Module.FileSize = File.Size(Module.Folder,Module.FileName)
		Module.LastModified = File.LastModified(Module.Folder,Module.FileName)
		
		Dim Line As String
		Dim LineNo As Int
		If File.Exists(Module.Folder,Module.FileName) Then
			Dim LineList As List = File.ReadList(Module.Folder,Module.FileName)
		Else
			Log("*** ERROR Module " & File.Combine(Module.Folder,Module.FileName) & " not found")
			Continue
		End If
		
		Line = LineList.Get(LineNo)
		Line = Line.Trim
		
		Do While Line.Trim <> "@EndOfDesignText@"
			If Line.StartsWith("Type=") Then
				Module.ModuleType = GetValue(Line)
			End If
			LineNo = LineNo + 1
			If LineNo >= LineList.Size Then Exit
			Line = LineList.Get(LineNo)
			Line = Line.Trim
		Loop
		
		Dim SubFound As Boolean
		
		Do While True
			If Line.ToLowerCase.StartsWith("private sub ") Or Line.ToLowerCase.StartsWith("public sub ") Or Line.ToLowerCase.StartsWith("sub ") Then
				Module.Subs.Add(GetSubName(Line))
				Module.CodeLines = Module.CodeLines + 1
				SubFound = True
			Else If Line.StartsWith("'") Then
				Module.CommentLines = Module.CommentLines + 1
				If Not(SubFound) Then
					Module.Description = Module.Description & CRLF & Line
				End If
			Else IF Line <> "" Then
				Module.CodeLines = Module.CodeLines + 1
			End If
			If Line.ToLowerCase.Contains("initializenewinstance(") Then
				Dim Class As String = GetJavaClass(Line)
				If Class <> ""  And Class <>"initializenewinstance(" And Module.JavaClasses.IndexOf(Class) = -1 Then Module.JavaClasses.Add(Class)
			End If
			If Line.ToLowerCase.Contains("initializestatic(") Then
				Dim Class As String = GetJavaClassStatic(Line)
				If Class <> "" And Class <> "initializestatic(" And Module.JavaClasses.IndexOf(Class) = -1 Then Module.JavaClasses.Add(Class)
			End If
			If Line.ToLowerCase.Contains("createevent") Then
				Dim Class As String = GetJavaEvent(Line)
				If Class <> "" And Module.JavaEvents.IndexOf(Class) = -1  Then Module.JavaEvents.Add(Class)
			Else If Line.ToLowerCase.Contains(".addchangelistener(") Then
				Dim Class As String = "javafx.beans.value.ChangeListener"
				Dim Result As String = Class & " on " & GetParam(Line,1)
				If Class <> "" And Module.JavaEvents.IndexOf(Result) = -1  Then Module.JavaEvents.Add(Result)
			Else If Line.ToLowerCase.Contains(".addeventfilter(") Then
				Dim Class As String = GetParam(Line,1)
				If Class <> "" And Module.JavaEvents.IndexOf(Class) = -1  Then Module.JavaEvents.Add(Class)
			Else If Line.ToLowerCase.Contains(".addeventhandler2(") Then
				Dim Class As String = GetParam(Line,1)
				If Class <> "" And Module.JavaEvents.IndexOf(Class) = -1  Then Module.JavaEvents.Add(Class)
			Else If Line.ToLowerCase.Contains(".addeventhandler(") Then
				Dim Class As String = GetParam(Line,1)
				If Class <> "" And Module.JavaEvents.IndexOf(Class) = -1  Then Module.JavaEvents.Add(Class)
			Else If Line.ToLowerCase.Contains(".createobject2(") Then
				Dim Class As String = GetJavaClass(Line)
				If Class <> "" And Class.ToLowerCase <> "createobject2("  And Module.JavaClasses.IndexOf(Class) = -1 Then Module.JavaClasses.Add(Class)
			Else If Line.ToLowerCase.Contains(".createobject(") Then
				Dim Class As String = GetJavaClassStatic(Line)
				If Class <> "" And Class.ToLowerCase <> "createobject("  And Module.JavaClasses.IndexOf(Class) = -1 Then Module.JavaClasses.Add(Class)
			Else If Line.ToLowerCase.StartsWith("type ") Then
				Module.Types.Add(Line.SubString(5))
			Else If Line.ToLowerCase.EndsWith("as form") Then
				Module.Forms.Add(GetWord(Line,1))
			End If
			LineNo = LineNo + 1
			If LineNo >= LineList.Size Then Exit
			Line = LineList.Get(LineNo)
			Line = Line.Trim
		Loop
		If Module.Description <> "" Then Module.Description = Module.Description.SubString(1)
	Next
End Sub

Private Sub GetSubName(Line As String) As String
	Dim Pos As Int = Line.ToLowerCase.IndexOf("sub") + 4
	Return Line.SubString(Pos)
End Sub

Private Sub GetValue(Line As String) As String
	Dim Pos As String = Line.IndexOf("=")
	If Pos > -1 Then
		Return Line.SubString(Pos + 1)
	Else
		Return ""
	End If
End Sub

Private Sub GetAttributeValue(Line As String) As String
	Dim Pos As Int = Line.ToLowerCase.IndexOf(":") + 1
	Return Line.SubString(Pos).Trim
End Sub

Private Sub NewProject(Folder As String, FileName As String) As ProjectType
	Dim P As ProjectType
	P.Initialize
	P.Folder = Folder
	P.ProjectFileName = FileName
	P.ProjectName = FileName.SubString2(0,FileName.LastIndexOf("."))
	P.Files.Initialize
	P.Modules.Initialize
	P.Libraries.Initialize
	P.Permissions.Initialize
	P.AddlJars.Initialize
	P.VirtualMachineArgs.Initialize
	Return P
End Sub

Private Sub NewModule(Folder As String, FileName As String) As ModuleType
	Dim M As ModuleType
	M.Initialize
	If FileName.StartsWith("|absolute|") Then
		FileName = FileName.SubString("|absolute|".Length)
		M.Folder = File.GetFileParent(FileName)
		M.ModuleName = FileName
		M.FileName = File.GetName(FileName) & ".bas"
	Else If FileName.StartsWith("|relative|") Then
		FileName = FileName.SubString("|relative|".Length)
		Dim FilePath() As String = Regex.Split("\\",FileName)
		For i = 0 To FilePath.Length - 1
			If FilePath(i) = ".." Then
				Folder = File.GetFileParent(Folder)
			End If
		Next
		FileName = FilePath(FilePath.Length - 1)
		M.Folder = Folder
		M.ModuleName = FileName
		M.FileName = FileName & ".bas"
	Else
		M.Folder = Folder
		M.ModuleName = FileName
		
		If FileName = "Main" Then
			M.FileName = FileName
		Else
			M.FileName = FileName & ".bas"
		End If
	End If
	M.Subs.Initialize
	M.Types.Initialize
	M.JavaClasses.Initialize
	M.JavaEvents.Initialize
	M.Forms.Initialize
	M.CodeLines = -1
	M.CommentLines = 0
	Return M
End Sub